'''
Employee.py
'''

from Pyro.EventService.Clients import Publisher

from GeneratedPy.GenEmployee import GenEmployee
from sndcs_common.DateTimeMixin import DateTimeMixin
from sndcs_common.SndcsExceptions import *
from sndcs_common.PyroProxyMixin import PyroProxyMixin
from sndcs.Config import config

class Employee(GenEmployee, DateTimeMixin, Publisher, PyroProxyMixin):

    def __init__(self):
        GenEmployee.__init__(self)
        Publisher.__init__(self)
        self.eventservice._release()

        # Get the Pyro namespace from the config
        self.namespace = config.get("pyro", "namespace", "sndcs")




    ### Status Functions ###

    def getActiveLaborDetails(self):
        """Convenience function - calls LaborHed.getActiveLaborDetails()"""
        if self.isClockedIn():
            return self.activeLaborHed().getActiveLaborDetails()

    def getNumberActiveLaborDetails(self):
        """Convenience function for laborHed.getNumberActiveLaborDtls"""
        if self.isClockedIn():
            return self.activeLaborHed().getNumberActiveLaborDtls()

    def getActivities(self):
        """Returns an array of dictionaries of the employees current activities in the form (activityDescription, activityType, laborDtlObject) where activityType is one of "production", "indirect", "setup", or "idle". laborDtlObject will be None if idle.
        Returns None if employee is not logged in (i.e. no active laborhed)"""
        if not self.activeLaborHed():
            return []
        current_activities = self.getActiveLaborDetails()
        if not current_activities:
            return [{"description":"*** IDLE ***", "type":"idle", "object":None},]
        else:
            activities = []
            for activity in current_activities:
                if activity.indirect():
                    type = "indirect"
                else:
                    if activity.jobType() == "Production":
                        type = "production"
                    elif activity.jobType() == "Setup":
                        type = "setup"
                    else:
                        #TODO: raise exception
                        type = ""

                activities.append({"description":activity.formattedDescription(), "type":type, "object":activity})
            return activities

    def isClockedIn(self):
        return bool(self.activeLaborHed())

    def isCurrentIndirectActivity(self, indirect):
        """Convenience function for laborHed.isCurrentIndirectActivity"""
        if self.isClockedIn():
            return self.activeLaborHed().isCurrentIndirectActivity(indirect)

    def isCurrentProductionActivity(self, operation, jobType="Production"):
        """Convenience function for laborHed.isCurrentProductionActivity"""
        if self.isClockedIn():
            return self.activeLaborHed().isCurrentProductionActivity(operation, jobType)
    
    def getActiveProductionLaborDetails(self):
        """Convenience function for laborHed.getActiveProductionLaborDetails"""
        if self.isClockedIn():
            return self.activeLaborHed().getActiveProductionLaborDetails()

    def getActiveProductionAndSetupLaborDetails(self):
        """Convenience function for laborHed.getActiveProductionAndSetupLaborDetails"""
        if self.isClockedIn():
            return self.activeLaborHed().getActiveProductionAndSetupLaborDetails()

    def getNumberActiveProductionLaborDetails(self):
        """Convenience function for laborHed.getNumberActiveProductionLaborDetails"""
        if self.isClockedIn():
            return self.activeLaborHed().getNumberActiveProductionLaborDetails()

    def getNumberActiveProductionAndSetupLaborDetails(self):
        """Convenience function for laborHed.getNumberActiveProductionLaborDetails"""
        if self.isClockedIn():
            return self.activeLaborHed().getNumberActiveProductionAndSetupLaborDetails()

    def isGanged(self):
        return self.getNumberActiveProductionLaborDetails() > 1

    def isSuspended(self):
        return bool( self.activeLaborHed().getSuspendedLaborDetails() )




    ### Clocking in and out ###

    def clockIn(self, start_date_time = None, terminal_id = None):
        if self.isClockedIn():
            raise SndcsEmployeeError("Employee is already clocked in.")
        from LaborHedFactory import LaborHedFactory # Couldn't put this up at the top because I was getting weird errors :X Some type of circular reference I think
        from Store import store
        hed = LaborHedFactory().new()
        if not start_date_time:
            start_date_time = self.now()
        hed.setStartStamp(start_date_time)
        hed.setStartTerminal(terminal_id)
        store.addObject(hed)
        self.setActiveLaborHed(hed)
        self.addToLaborHeds(hed)
        store.saveChanges()
        self.publish(self.namespace + "_clock_in", {"serialNum":self.serialNum(), "empId":self.empId(), "properName":self.properName(), "startStamp":start_date_time, "startTerminal":terminal_id})
        self.eventservice._release()

    def clockOut(self, end_date_time = None, terminal_id = None):
        if not self.isClockedIn():
            raise SndcsEmployeeError("Employee is not clocked in yet.")

        if not end_date_time:
            end_date_time = self.now()
            
        hed = self.activeLaborHed()
        hed.end(end_date_time, terminal_id)
        self.setActiveLaborHed(None)
        #self.addToLaborHeds(hed)
        from Store import store
        store.saveChanges()
        self.publish(self.namespace + "_clock_out", {"serialNum":self.serialNum(), "empId":self.empId(), "properName":self.properName(), "endStamp":end_date_time, "endTerminal":terminal_id, "laborHed":hed.serialNum()})
        self.eventservice._release()

    ### Departments ###
    def departments(self):
        list_departments = []
        for x in self.department():
            list_departments.extend([x.code()])

        return list_departments

    ### Adding and Deleting employees
    def delete(self):
        self.endAllLaborDtl()
        self.publish(self.namespace + "_employee_delete", {"serialNum":self.serialNum(),"empId":self.empId(), "properName":self.properName() })
        self.eventservice._release()

    def add(self,actionType="add"):
        from sndcs.Store import store
        store.saveChanges()
        self.publish(self.namespace + "_employee_%s" % actionType, {"serialNum":self.serialNum(),"empId":self.empId(), "properName":self.properName(), "departments":self.departments() })
        self.eventservice._release()

    ### Ending activities ###

    def endAllLaborDtl(self, terminal_id = None):
        """Convenience function - calls LaborHed.endAllLaborDtl()"""
        if self.isClockedIn():
            self.activeLaborHed().endAllLaborDtl(terminal_id = terminal_id)



    def stopAllActiveLaborDtls(self, terminal_id = None):
        if self.isClockedIn():
            self.activeLaborHed().endAllActiveLaborDtl(terminal_id = terminal_id)


    ### Suspending and resuming activites ###

    def suspendAllLaborDtl(self):
        """Convenience function - calls LaborHed.suspendAllLaborDtl()"""
        if self.isClockedIn():
            self.activeLaborHed().suspendAllLaborDtl()

    def resumeAllLaborDtls(self, terminal_id = None):
        """Convenience function for laborHed.resumeAllLaborDtls"""
        if self.isClockedIn():
            return self.activeLaborHed().resumeAllLaborDtls(terminal_id = terminal_id)






    ### Starting activities ###

    def startIndirect(self, indirect, terminal_id = None, global_indirect = False):
        from Indirect import Indirect
        assert isinstance(indirect, Indirect)
        if not self.isClockedIn():
            raise SndcsEmployeeError("Employee is not clocked in yet.")
        # If was already on this indirect return to idle, else start it
        if self.isCurrentIndirectActivity(indirect):
            if global_indirect: self.suspendAllLaborDtl()
            else: self.endAllLaborDtl(terminal_id = terminal_id)
        else:
            if global_indirect: self.suspendAllLaborDtl()
            else: self.endAllLaborDtl(terminal_id = terminal_id)
            dtl= self.activeLaborHed().startLaborDtl(indirect=indirect, jobType="Indirect", startTerminal=terminal_id)
            from Store import store
            store.saveChanges()

    def startProduction(self, operation, jobType = "Production", terminal_id = None):
        from Operation import Operation
        assert isinstance(operation, Operation)
        if not self.isClockedIn():
            raise SndcsEmployeeError("Employee is not clocked in yet.")
        # If was already on this operation (of same type Production/Setup) return to idle, else start it
        if self.isCurrentProductionActivity(operation, jobType):
            self.endAllLaborDtl(terminal_id = terminal_id)
        else:
            self.endAllLaborDtl(terminal_id = terminal_id)
            self.activeLaborHed().startLaborDtl(operation=operation, jobType=jobType, startTerminal=terminal_id)
            from Store import store
            store.saveChanges()








    ### Lunch and break ###

    def log_on_off_lunch_break(self, type=1, terminal_id = None):
        """type: lunch=1, break=2"""
        if not self.isClockedIn():
            raise SndcsEmployeeError("Employee is not clocked in yet.")
        from IndirectFactory import IndirectFactory
        from Store import store
        if type == 1:
            indirect = IndirectFactory().getIndirectByCode("LUNCH")
            other = IndirectFactory().getIndirectByCode("BREAK")
        elif type == 2:
            indirect = IndirectFactory().getIndirectByCode("BREAK")
            other = IndirectFactory().getIndirectByCode("LUNCH")

        if self.isCurrentIndirectActivity(other): # toggle lunch <--> break
            for current_activity in self.getActiveLaborDetails():
                current_activity.end(activity_percentage = 100.0, terminal_id = terminal_id)
            from LaborDtlFactory import LaborDtlFactory
            dtl = LaborDtlFactory().new(indirect=indirect, startTerminal = terminal_id)
            self.activeLaborHed().addToLaborDtls(dtl)
            store.saveChanges()
            if type == 1:
                self.publish(self.namespace + "_lunch_in", {"serialNum":self.serialNum(), "empId":self.empId(), "properName":self.properName(), "startStamp":self.now(), "startTerminal":terminal_id, "activity":{"serialNum":dtl.serialNum(), "indirect":indirect.code(), "description":indirect.description()}})
                self.eventservice._release()
            elif type == 2:
                self.publish(self.namespace + "_break_in", {"serialNum":self.serialNum(), "empId":self.empId(), "properName":self.properName(), "startStamp":self.now(), "startTerminal":terminal_id, "activity":{"serialNum":dtl.serialNum(), "indirect":indirect.code(), "description":indirect.description()}})
                self.eventservice._release()
        elif self.isCurrentIndirectActivity(indirect): # coming back
            for current_activity in self.getActiveLaborDetails():
                current_activity.end(activity_percentage = 100.0, terminal_id = terminal_id)
            self.resumeAllLaborDtls(terminal_id = terminal_id)
            store.saveChanges()
        else: # going to
            self.suspendAllLaborDtl()
            from LaborDtlFactory import LaborDtlFactory
            dtl = LaborDtlFactory().new(indirect=indirect, startTerminal = terminal_id)
            self.activeLaborHed().addToLaborDtls(dtl)
            store.saveChanges()
            if type == 1:
                self.publish(self.namespace + "_lunch_in", {"serialNum":self.serialNum(), "empId":self.empId(), "properName":self.properName(), "startStamp":self.now(), "startTerminal":terminal_id, "activity":{"serialNum":dtl.serialNum(), "indirect":indirect.code(), "description":indirect.description()}})
                self.eventservice._release()
            elif type == 2:
                self.publish(self.namespace + "_break_in", {"serialNum":self.serialNum(), "empId":self.empId(), "properName":self.properName(), "startStamp":self.now(), "startTerminal":terminal_id, "activity":{"serialNum":dtl.serialNum(), "indirect":indirect.code(), "description":indirect.description()}})
                self.eventservice._release()
            
            


    ### Start/Stop/Continue ###

    def startStopContinueLaborDtls(self, starts = None, stops = None, continues = None, terminal_id = None, jobType = "Production"):
        """Convenience function - calls LaborHed.startStopContinueLaborDtls()"""
        if self.isClockedIn():
            return self.activeLaborHed().startStopContinueLaborDtls(starts = starts, stops = stops, continues = continues, terminal_id = terminal_id,jobType = jobType)
